"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildUpdateDatastoreQuery = exports.onSubmitValidationRun = exports.buildDataStoreExpression = void 0;
const typescript_1 = require("typescript");
const helpers_1 = require("../../helpers");
const display_value_1 = require("./display-value");
const form_state_1 = require("./form-state");
const relationship_1 = require("./relationship");
const buildDataStoreExpression = (dataStoreActionType, importedModelName, hasManyFieldConfigs) => {
    if (dataStoreActionType === 'update') {
        return [
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('original'), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('query')), undefined, [typescript_1.factory.createIdentifier(importedModelName), typescript_1.factory.createIdentifier('id')]))),
            ], typescript_1.NodeFlags.Const)),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(importedModelName), typescript_1.factory.createIdentifier('copyOf')), undefined, [
                    typescript_1.factory.createIdentifier('original'),
                    typescript_1.factory.createArrowFunction(undefined, undefined, [
                        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('updated'), undefined, undefined, undefined),
                    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('assign')), undefined, [typescript_1.factory.createIdentifier('updated'), typescript_1.factory.createIdentifier('modelFields')])),
                    ], true)),
                ]),
            ]))),
        ];
    }
    // TODO: Many to Many update action, this condition dataStoreActionType === 'create' can be moved inside the functon
    if (hasManyFieldConfigs.length > 0 && dataStoreActionType === 'create') {
        return hasManyFieldConfigs
            .map((hasManyFieldConfig) => {
            var _a;
            const [, fieldConfigMetaData] = hasManyFieldConfig;
            if (((_a = fieldConfigMetaData.relationship) === null || _a === void 0 ? void 0 : _a.type) === 'HAS_MANY' &&
                fieldConfigMetaData.relationship.relatedJoinTableName) {
                return (0, relationship_1.buildManyToManyRelationshipCreateStatements)(importedModelName, hasManyFieldConfig);
            }
            return [];
        })
            .reduce((statements, statement) => {
            return [...statements, ...statement];
        }, []);
    }
    return [
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
            typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(importedModelName), undefined, [
                typescript_1.factory.createIdentifier('modelFields'),
            ]),
        ]))),
    ];
};
exports.buildDataStoreExpression = buildDataStoreExpression;
/**
  example: const validationResponses = await Promise.all(
    Object.keys(validations).reduce((promises, fieldName) => {
      if (Array.isArray(modelFields[fieldName])) {
        promises.push(
          ...modelFields[fieldName].map((item) =>
            runValidationTasks(fieldName, item, getDisplayValue[fieldName]),
          ),
        );
        return promises;
      }
      promises.push(runValidationTasks(fieldName, modelFields[fieldName], getDisplayValue[fieldName]));
      return promises;
    }, []),
  );
  if (validationResponses.some((r) => r.hasError)) {
    return;
  }
*/
const onSubmitValidationRun = (shouldUseGetDisplayValue) => {
    const getDisplayValueAccess = typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier(display_value_1.getDisplayValueObjectName), typescript_1.factory.createIdentifier('fieldName'));
    const runValidationTasksArgsForArray = [
        typescript_1.factory.createIdentifier('fieldName'),
        typescript_1.factory.createIdentifier('item'),
    ];
    const runValidationTasksArgs = [
        typescript_1.factory.createIdentifier('fieldName'),
        typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')),
    ];
    if (shouldUseGetDisplayValue) {
        runValidationTasksArgsForArray.push(getDisplayValueAccess);
        runValidationTasksArgs.push(getDisplayValueAccess);
    }
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('validationResponses'), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [
                typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('keys')), undefined, [typescript_1.factory.createIdentifier('validations')]), typescript_1.factory.createIdentifier('reduce')), undefined, [
                    typescript_1.factory.createArrowFunction(undefined, undefined, [
                        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('promises'), undefined, undefined, undefined),
                        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('fieldName'), undefined, undefined, undefined),
                    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                        typescript_1.factory.createIfStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Array'), typescript_1.factory.createIdentifier('isArray')), undefined, [
                            typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')),
                        ]), typescript_1.factory.createBlock([
                            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                                typescript_1.factory.createSpreadElement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')), typescript_1.factory.createIdentifier('map')), undefined, [
                                    typescript_1.factory.createArrowFunction(undefined, undefined, [
                                        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('item'), undefined, undefined, undefined),
                                    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, runValidationTasksArgsForArray)),
                                ])),
                            ])),
                            typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                        ], true), undefined),
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                            typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, runValidationTasksArgs),
                        ])),
                        typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                    ], true)),
                    typescript_1.factory.createArrayLiteralExpression([], false),
                ]),
            ]))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createIfStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('validationResponses'), typescript_1.factory.createIdentifier('some')), undefined, [
            typescript_1.factory.createArrowFunction(undefined, undefined, [
                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
            ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('hasError'))),
        ]), typescript_1.factory.createBlock([typescript_1.factory.createReturnStatement(undefined)], true), undefined),
    ];
};
exports.onSubmitValidationRun = onSubmitValidationRun;
const buildUpdateDatastoreQuery = (dataTypeName, recordName) => {
    // TODO: update this once cpk is supported in datastore
    const pkQueryIdentifier = typescript_1.factory.createIdentifier('id');
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('queryData'), undefined, undefined, typescript_1.factory.createArrowFunction([typescript_1.factory.createModifier(typescript_1.SyntaxKind.AsyncKeyword)], undefined, [], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                    typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('record'), undefined, undefined, typescript_1.factory.createConditionalExpression(pkQueryIdentifier, typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('query')), undefined, [typescript_1.factory.createIdentifier(dataTypeName), pkQueryIdentifier])), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createIdentifier((0, helpers_1.lowerCaseFirst)(dataTypeName)))),
                ], typescript_1.NodeFlags.Const)),
                typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression((0, form_state_1.getSetNameIdentifier)(recordName), undefined, [
                    typescript_1.factory.createIdentifier('record'),
                ])),
            ], true))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('queryData'), undefined, [])),
    ];
};
exports.buildUpdateDatastoreQuery = buildUpdateDatastoreQuery;
//# sourceMappingURL=cta-props.js.map