"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildManyToManyRelationshipCreateStatements = exports.buildRelationshipQuery = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const typescript_1 = require("typescript");
const form_state_1 = require("./form-state");
const react_studio_template_renderer_helper_1 = require("../../react-studio-template-renderer-helper");
const imports_1 = require("../../imports");
const buildRelationshipQuery = (relationship, importCollection) => {
    const { relatedModelName } = relationship;
    const itemsName = (0, form_state_1.getRecordsName)(relatedModelName);
    const objectProperties = [
        typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('type'), typescript_1.factory.createStringLiteral('collection')),
        typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier('model'), typescript_1.factory.createIdentifier(importCollection.getMappedAlias(imports_1.ImportSource.LOCAL_MODELS, relatedModelName))),
    ];
    return (0, react_studio_template_renderer_helper_1.buildBaseCollectionVariableStatement)(itemsName, typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('useDataStoreBinding'), undefined, [
        typescript_1.factory.createObjectLiteralExpression(objectProperties, true),
    ]));
};
exports.buildRelationshipQuery = buildRelationshipQuery;
const buildManyToManyRelationshipCreateStatements = (modelName, hasManyFieldConfig) => {
    const [fieldName, fieldConfigMetaData] = hasManyFieldConfig;
    const { relatedModelField, relatedJoinFieldName, relatedJoinTableName } = fieldConfigMetaData.relationship;
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(relatedModelField), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(modelName), undefined, [
                    typescript_1.factory.createIdentifier('modelFields'),
                ]),
            ]))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [
            typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(fieldName), typescript_1.factory.createIdentifier('reduce')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('promises'), undefined, undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier(relatedJoinFieldName), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                        typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                            typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(relatedJoinTableName), undefined, [
                                typescript_1.factory.createObjectLiteralExpression([
                                    typescript_1.factory.createShorthandPropertyAssignment(typescript_1.factory.createIdentifier(relatedModelField), undefined),
                                    typescript_1.factory.createShorthandPropertyAssignment(typescript_1.factory.createIdentifier(relatedJoinFieldName), undefined),
                                ], true),
                            ]),
                        ]),
                    ])),
                    typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                ], true)),
                typescript_1.factory.createArrayLiteralExpression([], false),
            ]),
        ]))),
    ];
};
exports.buildManyToManyRelationshipCreateStatements = buildManyToManyRelationshipCreateStatements;
//# sourceMappingURL=relationship.js.map